/*************************************************
* Montgomery Reducer Source File                 *
* (C) 1999-2004 The Botan Project                *
*************************************************/

#include <botan/montgom.h>
#include <botan/numthry.h>
#include <botan/mp_types.h>

namespace Botan {

/*************************************************
* Precompute values                              *
*************************************************/
MontgomeryReducer::MontgomeryReducer(const BigInt& mod) :
   ModularReducer(mod), modulus_times(256)
   {
   if(modulus.is_even())
      throw Invalid_Argument("MontgomeryReducer: Modulus must be odd");

   const u32bit n = modulus.bytes();

   R.set_bit(8 * n);
   RR = BigInt(BigInt::Power2, 2*n*8);
   RR %= modulus;

   M_inv = 256 - inverse_mod(modulus, 256).to_u32bit();

   for(u32bit j = 0; j != 256; j++)
      modulus_times[j] = modulus * j;

   mR = modulus * R;
   }

/*************************************************
* Convert to the Montgomery form                 *
*************************************************/
BigInt MontgomeryReducer::convert_in(const BigInt& x) const
   {
   if(x.is_negative())
      throw Invalid_Argument("MontgomeryReducer::convert_in: Invalid input");
   if(x >= modulus)
      return multiply(x % modulus, RR);
   return multiply(x, RR);
   }

/*************************************************
* Convert from the Montgomery form               *
*************************************************/
BigInt MontgomeryReducer::convert_out(const BigInt& x) const
   {
   return reduce(x);
   }

/*************************************************
* Montgomery Multiplication                      *
*************************************************/
BigInt MontgomeryReducer::multiply(const BigInt& x, const BigInt& y) const
   {
#if 0
   t1 = 0;
   const u32bit n = modulus.sig_words();
   for(u32bit j = 0; j != n; j++)
      {
      word u = (x.word_at(j) * y.word_at(0) + t1.word_at(0)) * M_inv;
      t2 = modulus * u;
      t1 += x.word_at(j) * y;
      t1 += t2;
      t1 >>= MP_WORD_BITS;
      }

   if(t1 > modulus)
      t1 -= modulus;

   return t1;
#else
   return reduce(x*y);
#endif
   }

/*************************************************
* Montgomery Reduction                           *
*************************************************/
BigInt MontgomeryReducer::reduce(const BigInt& x) const
   {
   if(x >= mR || x.is_negative())
      throw Invalid_Argument("MontgomeryReducer::reduce: Input invalid");

   t1 = x;

   const u32bit n = modulus.bytes();
   for(u32bit j = 0; j != n; j++)
      {
      byte x = t1.byte_at(j) * M_inv;
      t2 = modulus_times[x];
      t2 <<= (j*8);
      t1 += t2;
      }
   t1 >>= n * 8;

   if(t1 > modulus)
      t1 -= modulus;

   return t1;
   }

}
